package com.nisum.example.mongodb.service;

import com.nisum.example.mongodb.mapper.EmployeeMapper;
import com.nisum.example.mongodb.model.EmployeeDTO;
import com.nisum.example.mongodb.repository.EmployeeRepository;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

/**
 *Service class having methods for CURD operations
 * Methods having Mono<T> and Flux<T> as return types
 */
@Slf4j
@Service
public class EmployeeServiceImpl implements IEmployeeService {

    @Autowired
    private EmployeeRepository employeeRepository;

    @Autowired
    private EmployeeMapper employeeMapper;

    /**
     * perform create operation
     * @param employeeDTO
     * @return Mono
     */
    @Override
    public Mono<EmployeeDTO> insert(EmployeeDTO employeeDTO) {
        log.info("calling repository save method...");
        return employeeRepository.save(employeeDTO);
    }

    /**
     * perform fetch operation based on ID
     * @param id
     * @return Mono
     */
    @Override
    public Mono<EmployeeDTO> fetchById(Integer id) {
        log.info("Calling repository findById...");
        return employeeRepository.findById(id);

    }


    /**
     * perform fetch operation based on name
     * @param name
     * @return Flux
     */
    @Override
    public Flux<EmployeeDTO> fetchByName(String name) {
        log.info("Calling repository fetchByName");
        return employeeRepository.findByName(name)
                .map(employeeMapper::toDto);
    }

    @Override
    public Mono<EmployeeDTO> modifyById(Integer id) {

        return employeeRepository.findById(id);
    }

    /**
     * perform fetch all records
     * @return Flux
     */
    @Override
    public Flux<EmployeeDTO> fetchAll() {
        log.info("Calling repository findAll() method..");
        return employeeRepository.findAll();
    }

    /**
     * perform modify of object
     * @param empDTO
     * @return updated object of type Mono
     */
    @Override
    public Mono<EmployeeDTO> modify(EmployeeDTO empDTO) {
        log.info("Calling repository save() method...");
        return employeeRepository.save(empDTO);
    }

    /**
     * perform removeById based on id
     * @param id
     * @return Mono
     */
    @Override
    public Mono<Void> removeById(Integer id) {
        return employeeRepository.deleteById(id);
    }
}
